Require Export Program Classes.Equivalence Classes.Morphisms Setoid Relation_Definitions Classes.RelationClasses.
Set Implicit Arguments.
Generalizable All Variables.

(** * Introduction: proof irrelevant equivalence *) 

(** This module defines an equivalence [=pi=] between dependent
pairs. Dependently typed pairs consist of a term that is a value, and
a second one whose type depends on the first component. Often the
second one being usually a proof of a property of the first
component. The equivalence =pi= ignores the proof and identifies
dependent pairs with their first component. In specific cases, for
example if the property is an equality on a type with decidable
equality, it is possible to show that this relation is equivalent to
[=]. *)

(** * Equivalence definition, notation and properties *)

(** Equivalence definition.  *)
Definition  proofirrel_eq (A:Type)(P:A->Prop)(x y : {x : A | P x}) :=
  projT1 x = projT1 y.
Implicit Arguments proofirrel_eq [A P].
Hint Unfold proofirrel_eq.

(** Equivalence notation.  *)
Notation  " f =pi= g " := (proofirrel_eq f g) (at level 60). 

(** [proofirrel_eq] is an equivalence relation. *)
Program Instance proofirrel_equiv (A:Type)(P:A->Prop) : 
  Equivalence (proofirrel_eq(A:=A)(P:=P)).
Solve All Obligations using (auto with * || congruence).

