Require Import Coq.Lists.List.
Import ListNotations.
Require Import ProgramCalculation.theories.Algebra.Algebra.
Require Import ProgramCalculation.library.Notations.

(** * Usual BMF notations for functions defined in [Coq.Lists.List] *)

Notation "f ✶" := (map f)(at level 20) : BMF_scope.
Notation "(✶)" := (fun f l =>map f l): BMF_scope.
Notation "# x" := (length x)(at level 20) : BMF_scope.
Notation "(#)" := (fun x=>length x) : BMF_scope.
Notation "p ◁" := (filter p)(at level 20) : BMF_scope.
Notation "(◁)" := (fun p l=>filter p l): BMF_scope.
Notation "[⋅]" := (fun a =>[a]) : BMF_scope.
Notation "(++)" := (fun x y=>x ++ y) : BMF_scope.

Definition K {A:Type}(a:A){B:Type} := fun (b:B) => a.
Hint Unfold K.

(** * Definition and notation for reduction *)

Generalizable All Variables.

Definition reduce `(op:A -> A -> A) `{m: Monoid A op e} := 
  fun l => fold_left op l e. 
Notation "op /" := (reduce op)(at level 20) : BMF_scope.
Hint Unfold reduce.

(** By default the [BMF_scope] is opened. *)
Open Scope BMF_scope.

(** Examples *)

Section Examples.

  Require Import NPeano.

  Eval vm_compute in (+) / (S ✶ (odd ◁ [0;1;2;3;4;5;6;7;8;9;10])).
  
  Eval vm_compute in (#) ✶ ([⋅] ✶ [55;3;17]).

End Examples.